%Produces PSF images of Airy and Pixel Aperture (100% FF, Pitch on the side)
%Models PSFs, Computes DFTs, Multiplies them together and verifies model
%works

extendedPlot = false;
pitch = 5;
side = 1024;                            %size of image on the side
%% Units
lambdaN = 0.00053*16;                   %mm
x = -side/2:side/2-1;                   %lp/imgWidth
x = x / side;                           %lp/samples = lp/iw / (samples/iw)
[X,Y] = meshgrid(x);

%%
%Airy Model Spatial Domain
r = sqrt(X.^2 + Y.^2);
airyImg = (2*besselj(1,pi*r/lambdaN)./(pi*r/lambdaN)).^2;   %intensity
airyImg(isnan(airyImg)) = 1;                    %fix origin
%Airy Model Frequency Domain
rho = r * lambdaN * side;
airyMTF = 2/pi*(acos(rho)-rho .* sqrt((1-(rho).^2)));
airyMTF(rho>1) = 0;

%Pixel Aperture Model Spatial Domain
sqImg = zeros(side);
a = floor(side/2)+1 - floor(pitch/2);
sqImg(a:a+pitch-1,a:a+pitch-1) = 1;
%Px Aperture Model Frequency Domain
sqMTF = abs(sinc(X * pitch)) .* abs(sinc(Y * pitch));

%%
%ROI MTF - Yes, the size of the image affects MTF
roiMTF = abs(sinc(X)) .* abs(sinc(Y));

%Airy 'Measured' Frequency Domain
airyDFT = abs(fftshift(fft2(airyImg)));
airyDFT = airyDFT / max(max(airyDFT));

%Px Aperture Measured Frequency Domain
sqDFT = abs(fftshift(fft2(sqImg))) .* roiMTF;
sqDFT = sqDFT / pitch^2;                            %normalized

%%
%Airy*Px Model
totMTF = airyMTF .* sqMTF;

%Airy*Px Measured
totDFT = airyDFT .* sqDFT;

%%
%Horizontal Slice
horDFT = totDFT(side/2+1,:);                        %Measured
horMTF = airyMTF(side/2+1,:) .* sinc(x*pitch);      %Modeled

%Diagonal Slice
clear diagMTF diagDFT diagAiryMTF
for i = 1:length(totDFT)
    diagDFT(i) = totDFT(i,i);                       %Measured
    diagAiryMTF(i) = airyMTF(i,i);                  
end

diagMTF = diagAiryMTF .* sinc(x*pitch).^2;          %Modeled
        %projected rect looks like triangle at 45deg, FT = sinc^2

%% Print stuff
%Horizontal Slice
figure; plot(x*pitch,horDFT,'.'); hold on; plot(x*pitch,horMTF); grid on;
airySampling; plot(x1,sampFT(103,:),'k')
xlim([0 1])
legend('Measured','Modeled','Sampled'); ylabel('MTF'); xlabel('cycles/pixel')
title('Horizontal Slice off 2D MTF, Diffraction + Px Aperture')

%Diagonal Slice
figure; plot(x*pitch,horDFT,'--'); hold on; plot(x*sqrt(2)*pitch,diagMTF)
plot(x*sqrt(2)*pitch,diagDFT,'.'); grid on; ylabel('MTF'); xlabel('cycles/pixel')
legend('Horizontal Slice Measured','Diagonal Slice Modeled','Diagonal Slice Measured')
title('Diagonal Slice off 2D MTF, Diffraction + Px Aperture')

if extendedPlot
    figure; mesh(X*pitch,Y*pitch,totMTF); title('Diff*Px Computed')
    figure; mesh(X*pitch,Y*pitch,totDFT); title('Diff*Px Measured')
    %Horizontal
    figure; plot(x*pitch,sqMTF(513,:)); hold on; plot(x*pitch,sqDFT(513,:),'.')
    title('Central slice 2D MTF')
    figure; plot(x*pitch,sqMTF(513,:)); hold on; plot(x*pitch,sqDFT(513,:),'.'); xlim([0.45 0.5])
    title('Central slice 2D MTF, Zoomed')
end



